function  addprx(pth, files)
global pf

if not(iscell(files))
    files = {files};
end

for k = 1:numel(files)
    dat = readprx(pth, fullfile(pth, files{k}));
    if ischar(dat)
        h = errordlg(dat);
        waitfor(h);
        continue
    end
    dat.name = genvarname(dat.name, fields(pf.prx));
    if isfield(pf.prx,dat.name)
        msgbox([dat.name ' was already in memory...replacing'],'Duplicate File Name','help');
        pause(2)
    end
    pf.prx.(dat.name) = dat;
end

end

function data = readprx(fpath, file)

% reads proximity data from file along with node labels (if available)
% output is a structure with the following fields:
% data.file: filename
% data.distances: true for distance, dissimilarity, false for similarity
% data.nnodes: number of nodes (n)
% data.comment: short description
% data.minprx: the minimum value allowed, smaller are considered infinite distances
% data.maxprx: the maximum value allowed, larger are condisered infinite distances
% data.dis:  the distance matrix
% data.ndata: number of datapoints (not counting infinite distance or 0 similarity)
% data.coh: the coherence of the data if it can be computed else NaN
% data.directed: whether the data are asymmetric
% data.loops: whether distances from a node to itself are not zero
% Values missing or outside range are infinite for data.dis

% open file and check for 'data'
fid = fopen(file);
if fid < 0
    data = ['Error: cannot open ' file];
    return
end
t = lower(fgetl(fid));
if ~isin(t,'data')
    data = ['Error: ' file ' does not appear to be a proximity data file.'];
    fclose(fid);
    return
end
data.contents = 'prx';
data.file = file;
[~, data.name] = fileparts(file);
data.name = strtok(data.name,'.');

% distances or similarities
t = lower(fgetl(fid));
if isin(t,'dis')
    data.distances = true;
else
    data.distances = false;
end

% nnodes, terms, comment, min, max
data.nnodes = getnum(fid);
if isempty(data.nnodes)
    data = ['Error: cannot get number of items from ' file];
    fclose(fid);
    return
end

data.nodes = getnodes(fpath, data.name, data.nnodes);

data.comment = string(fgetl(fid)); 

data.minprx = getnum(fid);
if isempty(data.minprx)
    data = ['Error: cannot get minimum value from ' file];
    fclose(fid);
    return
end
data.maxprx = getnum(fid);
if isempty(data.maxprx)
    data = ['Error: cannot get maximum value from ' file];
    fclose(fid);
    return
end

% data shape and number of data values
nvals = data.nnodes^2;
t = lower(fgetl(fid));
if isin(t,'coord') || isin(t,'attrib') || isin(t,'featur')
    data.distances = true;
    data.shape = 'coord';
    ndims = getnum(fid);
    nvals = ndims.*data.nnodes;
    metric = lower(fgetl(fid));
    if isin(metric,'euclid')
        normval = 2;
    elseif isin(metric,'city')
        normval = 1;
    elseif isin(metric,'domin')
        normval = Inf;
    else
        normval = 2;
    end
    hamming = isin(metric,'hamming');
    standardize = isin(metric,'standard');
    cosine = isin(metric,'cosine');
elseif isin(t,'lower')
    data.shape = 'lower';
    nvals = (nvals - data.nnodes)/2;
elseif isin(t,'upper')
    data.shape = 'upper';
    nvals = (nvals - data.nnodes)/2;
elseif isin(t,'matrix')
    data.shape = 'matrix';
elseif isin(t,'list')
    data.shape = 'list';
    npairs = getnum(fid);
    nvals = 3*npairs;
    t = lower(fgetl(fid));
    if isin(t,'non') || isin(t,'as') || isin(t,'uns') || isin(t,'dir')
        data.directed = true;
    else
        data.directed = false;
    end
else
    data = ['Error: unknown data shape in ' file];
    fclose(fid);
    return
end

% read data
dat = textscan(fid,'%f');
fclose(fid);
dat = dat{1};
if numel(dat) ~= nvals
    data =  ['Error in: ' file ' Expected ' num2str(nvals) ' data values, found ' num2str(numel(dat))];
    return
end

% make room for distance data

prx = Inf(data.nnodes);

switch data.shape
    case 'coord'
        data.minprx = Inf;
        data.maxprx = -Inf;
        dat = reshape(dat,ndims,data.nnodes)';
        if standardize
            for k = 1:data.nnodes
                dat(k,:) = dat(k,:)/norm(dat(k,:),1);
                %dat = dat./repmat(sum(dat,2),1,size(dat,2));
            end
        end
        for r = 2:data.nnodes
            for c = 1:(r-1)
                if hamming
                    d = sum(dat(r,:) ~=  dat(c,:));
                elseif cosine
                    warning off all
                    d = 1 - cosang(dat(r,:),dat(c,:));
                    warning on all
                else
                    d = norm(dat(r,:) - dat(c,:),normval);
                end
                if abs(d) < 1e-14
                    d = 0;
                end
                if isnan(d)
                    d = 0;
                end
                prx(r,c) = d;
                prx(c,r) = d;
                data.minprx = min(d,data.minprx);
                data.maxprx = max(d,data.maxprx);
            end
        end
        if ndims < 3
            dat(data.nnodes,3) = 0;
        end
        data.view.coord = dat(:,1:3);
        data.directed = false;
        data.loops = false;
    case 'lower'
        k = 1;
        for r = 2:data.nnodes
            for c = 1 : (r-1)
                d = prx2dis(dat(k),data);
                prx(r,c)=d;
                prx(c,r)=d;
                k=k+1;
            end
        end
        data.directed = false;
        data.loops = false;
    case 'upper'
        k = 1;
        for r = 1:data.nnodes
            for c = (r+1):data.nnodes
                d = prx2dis(dat(k),data);
                prx(r,c)=d;
                prx(c,r)=d;
                k=k+1;
            end
        end
        data.directed = false;
        data.loops = false;
    case 'matrix'
        prx = reshape(dat,data.nnodes,data.nnodes);
        prx = prx'; %matlab moves down columns rather than across rows
        prx = prx2dis(prx,data);
        data.directed = ~all(all(prx == prx'));
        data.loops = ~all(prx(1) == diag(prx));  % no loops if all diagonal elements are equal
    case 'list'
        prx = forcediag(prx, 0);
        data.loops = false;
        dat = reshape(dat,3,npairs);
        for k = 1:npairs
            dd = dat(3,k);
            r = dat(1,k);
            c = dat(2,k);
            d = prx2dis(dd,data);
            prx(r,c) = d;
            if r == c
                data.loops = true;
            end
        end
        if ~data.directed %|| forcesym
            prx = min(prx,prx');
            data.directed = false;
        end
end

% finish up

if ~data.loops
    prx(logical(eye(data.nnodes))) = 0;
end
data.dis = prx;
data.coh = coherence(prx,data);

[data.stddev, data.ndata] = getstats(data);
end


function r = getnum(fid)
t = lower(fgetl(fid));
r = textscan(t,'%f');
r = r{1};
if iscell(r)
    r = r{1};
end
end

function cosa = cosang(v1,v2)
cosa = sum(v1.*v2)./(norm(v1)*norm(v2));
end

function nodes = getnodes(fpath, name,n)
% nodes = getnodes(name,n)
% tries to locate n terms for nodes in various files
% returns node numbers if file cannot be found
tfile = gettermfile(string(fpath),string(name));
nodes = readterms(tfile);
if length(nodes) == n
    return
else
    nodes = numbernodes(n);
end
end

function tfile = gettermfile(fpath, name)
global pf
tfile = fullfile(fpath,name) + ".trm.txt";
if exist(tfile,"file")
    return
end
tfile = fullfile(pf.projectdir,name) + ".trm.txt";
if exist(tfile,"file")
    return
end
tfile = fullfile(fpath,name) + ".trm";
if exist(tfile,"file")
    return
end
tfile = fullfile(pf.projectdir,name) + ".trm";
if exist(tfile,"file")
    return
end
tfile = fullfile(fpath,"terms.txt");
if exist(tfile,"file")
    return
end
tfile = fullfile(pf.projectdir,"terms.txt");
if exist(tfile,"file")
    return
end
end

function dis = prx2dis( prx, data )
%prx2dis returns distances from proximities
dis = prx;
if ~data.distances
    dis = data.maxprx - dis + data.minprx;
end
dis(prx < data.minprx | prx > data.maxprx) = Inf;
end




