function [view, fighandle] = displaynet(netname)
% net is a net structure
% view is a view structure

global pf

net = pf.net.(netname);

switch pf.pref.nodepositions
    case 1 %'Prev'
        view = net.view;
    case 2 %'New '
        view.coord = [];
        view.camera = struct;
    case 3 %'Node'
        if net.nnodes == length(pf.lastview.coord)
            view = pf.lastview;
        else
            view.coord = [];
            view.camera = struct;            
        end
end

fighandle = figure;

% Label the figure 
set(fighandle,'Name',netname,...
    'NumberTitle',    'off',...
    'Userdata',netname);

% Set the camera if available
if ~isempty(fieldnames(view.camera))
    setcamera(fighandle,view.camera);
end

view.camera = getcamera(fighandle);

% Set up node labels
labels = net.nodes;
if isempty(labels)
    labels = numbernodes(net.nnodes);
end
net.nodes = labels;

% Get new coordinates if necessary
if isempty(view.coord)
    view.coord = makecoord(net,pf.pref.dims);
    if isempty(view.coord)
        delete(fighandle)
        return     
    end
    view.camera = struct;
end

% draw the net

set(fighandle,'Color','white');

axis;
xlim([0 1]);
ylim([0 1]);
zlim([0 1]);
axis off

links = net.adj;
labels = net.nodes;
n = net.nnodes;
if length(labels) ~= n
    labels = numbernodes(n);
end

v = net.links(:,3);
if pf.pref.slider && numel(v) > 0 && ~all(v(1) == v)
    uicontrol(fighandle,'Style','slider',...
        'Max',1,'Min',0,'Value',0,...
        'SliderStep',[0.02 0.06],...
        'Position',[0 0 15 300],...
        'Callback',@slider_callback);
end

% draw links
coord = view.coord;
coord(:,1) = coord(:,1) - min(coord(:,1));  % make x min 0
if max(coord(:,1)) ~= 0
    coord(:,1) = coord(:,1)/max(coord(:,1)); % make x max 1
end
coord(:,2) = coord(:,2) - min(coord(:,2));
if max(coord(:,2)) ~= 0
    coord(:,2) = coord(:,2)/max(coord(:,2));
end
if max(coord(:,3)) ~= 0
    coord(:,3) = coord(:,3) - min(coord(:,3));
    coord(:,3) = coord(:,3)/max(coord(:,3));
end

linkhandles = zeros(net.nlinks,1);
if net.directed
    pf.pref.dims = 2;
    coord(:,3) = 0;
    pf.pref.box = 'none';
    arrowhandles = linkhandles;
else
    pf.pref.box = 'white';
    arrowhandles = [];
end
if pf.pref.dims == 3
    cameratoolbar(fighandle);
end

hold on

% add legend

ltypes = [];
legh = [];
for k = 1:net.nlinks
    row = net.links(k,1);
    col = net.links(k,2);
    if row ~= col
        h = drawlink(net.directed,coord([row;col],:),net.linktypes(row,col),pf.linespec);
        linkhandles(k) = h(1);
        if net.directed
            arrowhandles(k) = h(2);
        end
        
        set(linkhandles(k),...
            'UserData',      [row, col, net.dis(row,col)],...
            'ButtonDownFcn', @link_callback);

        tt = net.linktypes(row,col);
        if ~ismember(tt,ltypes)
            legh = [legh h(1)]; %#ok<AGROW>
            ltypes = [ltypes tt]; %#ok<AGROW>
        end
    end
end

legendhandle = [];
if length(ltypes) < 25 && length(ltypes) > 1 %#ok<*ISMT>
    [ltypes, si] = sort(ltypes);
    
    legh = legh(si);
    
    leg = net.linktypenames(ltypes);
    leg(legh == 0) = [];
    legh(legh == 0) = [];
    if length(legh) > 1
        warning off all
        for i = 1:length(leg)
            t = leg{i};
            t(t == '_') = ' ';
            leg{i} = t;
        end
        legendhandle = legend(legh, leg,'Location','Best');
        legendhandle.AutoUpdate = 'off';
        warning on all
    end
end

% Add the node labels

nodehandles = zeros(n,1);
for k = 1:n
    if links(k,k) % border signals loops
        border = 'blue';
    else
        border = 'none';
    end
    nodehandles(k) = drawnode(k,labels{k},coord(k,:),border);
end

view.coord = coord;
view.dims = pf.pref.dims;
view.camera = getcamera;
view.linkhandles = linkhandles;
view.arrowhandles = arrowhandles;
view.nodehandles = nodehandles;
view.legendhandle = legendhandle;

net.view = view;
pf.net.(netname) = net;
pf.lastview = view;


end

% helper functions ********************************************************
function nodetexthandle = drawnode(nodenum,node,coord,border)
global pf
nodetexthandle = text(coord(1),coord(2),coord(3),parse(node));
FontSize = 10;
if ~ispc
    FontSize = 12;
end
set(nodetexthandle,...
    'FontName',           'Arial',...
    'FontSize',           FontSize,...
    'Color',              'blue',...
    'BackGroundColor',     pf.pref.box,...
    'horizontalalignment', pf.pref.align,...
    'UserData',            nodenum,...
    'ButtonDownFcn',       @node_callback);
if ~ isempty(border)
    set(nodetexthandle,'EdgeColor',border);
end
end


function setcamera(fig,camera)
% setcamera(fig,camera)
% sets the camera parameters of fig using camera structure
if ishandle(fig)
    a = get(fig,'CurrentAxes');
    if ishandle(a)
        campos(a,camera.pos);
        camtarget(a,camera.target);
        camup(a,camera.up);
        camva(a,camera.va);
    end
end
end

