function net = makenet(how, source)
% Net generation
%
%   calling form:
%     net = makenet(how, source)
%
%  how: a structure with fields containing applicable parameters including:
%       method: 'pfn'  (or 'nn' or 'nnc' or 'th' or 'oldpf' or 'mrg')
%     newrange: [NaN NaN] or new min and max
%            q: n-1
%            r: Inf
%   nneighbors: 1      1 through 5
%      threshn: 25     the number of desired links
%        links: matrix of links to be put into network structure
%
% source: a structure with the following fields:
%
%          name: 'psy'
%          file: 'C:\my\RS\psy.prx.txt'
%        nnodes: 25
%         nodes: {25x1 cell}
%        minprx: 0
%        maxprx: 8
%         loops: 0    boolean
%      directed: 0    boolean
%           dis: [25x25 double]
%
%output:
% % net: a structure with the following fields
%          contents: 'net'
%              name: 'psy'
%            nnodes: 25
%             nodes: {25x1 cell}
%          decimals: 1
%            minprx: 0
%            maxprx: 8
%             loops: 0
%          directed: 1
%                 q: NaN
%                 r: NaN
%              type: 'nnc'
%         linktypes: [25x25 double]
%     linktypenames: {'nn'  'nnc'}
%               adj: [25x25 logical]  
%             links: [25x3 double]   row col weight for links
%       ncomponents: 1
%        components: {[1x25 double]}
%            nlinks: 33
%            nloops: 0
%         connected: 1  boolean
%              view: [1x1 struct]
%            pfname: 'psy_nnc1'

net = source; % fields are copied to the network structure
for f = {'coh','size','decimals','shape','file','distances',...
        'stddev','ndata','minprx', 'maxprx'} %some are deleted
    if isfield(net,f{1})
        net = rmfield(net,f{1});
    end
end

net.contents = 'net';

net.q = NaN;
net.r = NaN;

net.type = how.method;

switch how.method(1)
    case 'n'
        net.q = nan;
        net.r = nan;
        [net.adj, net.linktypes, net.linktypenames] = ...
            nearestneighbor(net.dis);
        net.directed = true;
   
    case 't'
        net.q = nan;
        net.r = nan;
        
        dis = source.dis;
        how.threshn = how.threshn + how.threshn.*(~source.directed);
        if how.threshn > source.nnodes^2
           how.threshn = source.nnodes - 1; 
        end
        if ~source.loops
            dis(logical(eye(source.nnodes))) = inf;
        end
        half = logical(tril(ones(source.nnodes)));
        if source.directed
            v = sort(net.dis(:));
        else
            v = sort(net.dis(half));
        end
        thresh = v(how.threshn);
        net.adj = (dis <= thresh) & (dis < Inf);

        net.linktypes = double(net.adj);
        net.linktypenames = {'tn'};
        net.threshold = thresh;
 
    case 'p'
        net.r = how.r;
        if how.q >= (net.nnodes - 1)
            net.q = net.nnodes - 1;
        else
            net.q = how.q;
        end        
        dis = source.dis;
        if net.q >= (net.nnodes - 1)
            [~, net.adj] = floyd(dis,net.r); % only for q = n-1
        else
            [~, net.adj] = pfall(dis,net.q,net.r);
        end
        net.linktypes = double(net.adj);
        net.linktypenames = {'pfn'};
    case 'm'
        net.q = NaN;
        net.r = NaN;
end

if ~net.loops
    net.adj = forcediag(net.adj,false);
end
net.nlinks = nnz(net.adj);
net.nloops = nnz(diag(net.adj));
if ~net.directed
    net.nlinks = net.nloops + ((net.nlinks - net.nloops) / 2);
end

net.links = getlinks(net);

[net.ncomponents, net.components] = components(net.adj);
net.connected = net.ncomponents == 1;
net.view.coord = [];
net.view.camera = struct;
end


function [mindis, adj, lastq] = pfall(dis,q,r)
% [mindis links lastq] = pfall(dis,q,r)
% dis      | matrix with distances for all pairs of nodes (proximity data)
% q        | value of the q-parameter
% r        | value of the r-parameter
% mindis   | matrix of minimum distances between nodes
% adj      | links in PFnet(q,r)
% lastq    | the last value of q needed to reach mindis
% general PFNET generation algorithm based on Dijkstra's method

n = length(dis);
q = min(q,n-1);
mindis = dis;
pass = 1;
changed = true;

while changed && (pass < q)
    pass = pass + 1;
    changed = false;
    m = mindis;
    for row = 1 : n
        for col = 1 : n
            for ind = 1 : n
                %                indirect1 = minkowski(dis(row,ind),m(ind,col),r);
                %                indirect2 = minkowski(m(row,ind),dis(ind,col),r);
                %                norm is faster and correct with the test at the end of the function
                indirect1 = norm([dis(row,ind),m(ind,col)],r);
                indirect2 = norm([m(row,ind),dis(ind,col)],r);
                indirect = min(indirect1,indirect2);
                if indirect < mindis(row,col)
                    mindis(row,col) = indirect;
                    changed = true;
                end
            end
        end
    end

end %while

lastq = pass;
adj = (dis < Inf) & (abs(mindis - dis) < 1e-14);

return

end


function [adj, linktypes, linktypenames] = nearestneighbor(dis)
% links = nearestneignbor(sim, numn, connect)
% dis     | distance matrix
% adj     | boolean matrix indicating links
% Defines a directed network in which each node points to its
% nearest neighbor.  Usually produces disconnected networks.

sim = 1./dis;

nnodes = length(sim);
linktypes = zeros(nnodes);
sim(logical(eye(nnodes))) = 0;  % a node cannot be the nearest neighbor to itself
adj = false(nnodes);

% maximum of each row (nearest = most similar)
rowmax = max(sim, [], 2);
for k = 1:nnodes
    if rowmax(k)                                 % make sure the maximum similarity is not zero
        adj(k,rowmax(k) == sim(k,:)) = true;     % store links in locations of maximum
        linktypes(k,rowmax(k) == sim(k,:)) = 1;  % link type is nearest neighbor
    end
end

linktypenames = {'nn'};

end


